@@ -43,6 +43,7 @@ G_BEGIN_DECLS
 typedef struct _VinoServer        VinoServer;
 typedef struct _VinoServerClass   VinoServerClass;
 typedef struct _VinoServerPrivate VinoServerPrivate;
+typedef struct _VinoClient        VinoClient;
 
 typedef enum
 {
@@ -76,13 +77,14 @@ void                 vino_server_set_prompt_enabled       (VinoServer     *serve
 gboolean             vino_server_get_prompt_enabled       (VinoServer     *server);
 void                 vino_server_set_view_only            (VinoServer     *server,
 							   gboolean        view_only);
+gboolean             vino_server_get_view_only            (VinoServer     *server);
 gboolean             vino_server_get_use_alternative_port (VinoServer     *server);
 void                 vino_server_set_use_alternative_port (VinoServer     *server,
 							   gboolean        use_alternative_port);
 int                  vino_server_get_alternative_port     (VinoServer     *server);
 void                 vino_server_set_alternative_port     (VinoServer     *server,
 							   int             alternative_port);
-gboolean             vino_server_get_view_only            (VinoServer     *server);
+int                  vino_server_get_port                 (VinoServer     *server);
 void                 vino_server_set_local_only           (VinoServer     *server,
 							   gboolean        local_only);
 gboolean             vino_server_get_local_only           (VinoServer     *server);
@@ -95,6 +97,13 @@ VinoAuthMethod       vino_server_get_auth_methods         (VinoServer     *serve
 void                 vino_server_set_vnc_password         (VinoServer     *server,
 							   const char     *vnc_password);
 G_CONST_RETURN char *vino_server_get_vnc_password         (VinoServer     *server);
+void                 vino_server_set_lock_screen          (VinoServer     *server,
+							   gboolean        lock_screen);
+gboolean             vino_server_get_lock_screen          (VinoServer     *server);
+
+
+G_CONST_RETURN char *vino_client_get_hostname (VinoClient *client);
+void                 vino_client_disconnect   (VinoClient *client);
 
 G_END_DECLS
 
